@@ -51,6 +51,7 @@ from .types import (
     DEFAULT_API_CONNECT_OPTIONS,
     NOT_GIVEN,
     APIConnectOptions,
+    FlushSentinel,
     NotGiven,
     NotGivenOr,
 )
@@ -75,6 +76,7 @@ from .voice import (
     UserStateChangedEvent,
     avatar,
     io,
+    room_io,
 )
 from .voice.background_audio import AudioConfig, BackgroundAudioPlayer, BuiltinAudioClip, PlayHandle
 from .voice.room_io import RoomInputOptions, RoomIO, RoomOutputOptions
@@ -91,8 +93,7 @@ from .voice.run_result import (
     mock_tools,
 )
 from .worker import (
-    SimulateJobInfo,
-    Worker,
+    AgentServer,
     WorkerOptions,
     WorkerPermissions,
     WorkerType,
@@ -113,7 +114,7 @@ def __getattr__(name: str) -> typing.Any:
 
 __all__ = [
     "__version__",
-    "Worker",
+    "AgentServer",
     "WorkerOptions",
     "WorkerType",
     "WorkerPermissions",
@@ -127,6 +128,7 @@ __all__ = [
     "function_tool",
     "ChatContext",
     "ChatItem",
+    "room_io",
     "RoomIO",
     "RoomInputOptions",
     "RoomOutputOptions",
@@ -169,7 +171,7 @@ __all__ = [
     "BuiltinAudioClip",
     "AudioConfig",
     "PlayHandle",
-    "SimulateJobInfo",
+    "FlushSentinel",
     "io",
     "avatar",
     "cli",
