@@ -5,7 +5,7 @@ from dataclasses import dataclass
 from typing import TYPE_CHECKING
 
 from ... import llm, stt, tts, vad
-from ...llm.tool_context import ToolError, function_tool
+from ...llm.tool_context import ToolError, ToolFlag, function_tool
 from ...types import NOT_GIVEN, NotGivenOr
 from ...voice.agent import AgentTask
 from ...voice.events import RunContext
@@ -27,8 +27,10 @@ class GetEmailResult:
 class GetEmailTask(AgentTask[GetEmailResult]):
     def __init__(
         self,
+        extra_instructions: str = "",
         chat_ctx: NotGivenOr[llm.ChatContext] = NOT_GIVEN,
         turn_detection: NotGivenOr[TurnDetectionMode | None] = NOT_GIVEN,
+        tools: NotGivenOr[list[llm.FunctionTool | llm.RawFunctionTool]] = NOT_GIVEN,
         stt: NotGivenOr[stt.STT | None] = NOT_GIVEN,
         vad: NotGivenOr[vad.VAD | None] = NOT_GIVEN,
         llm: NotGivenOr[llm.LLM | llm.RealtimeModel | None] = NOT_GIVEN,
@@ -58,9 +60,11 @@ class GetEmailTask(AgentTask[GetEmailResult]):
                 "If the email is unclear or invalid, or it takes too much back-and-forth, prompt for it in parts: first the part before the '@', then the domain—only if needed. \n"
                 "Ignore unrelated input and avoid going off-topic. Do not generate markdown, greetings, or unnecessary commentary. \n"
                 "Always explicitly invoke a tool when applicable. Do not simulate tool usage, no real action is taken unless the tool is explicitly called."
+                + extra_instructions
             ),
             chat_ctx=chat_ctx,
             turn_detection=turn_detection,
+            tools=tools or [],
             stt=stt,
             vad=vad,
             llm=llm,
@@ -75,12 +79,7 @@ class GetEmailTask(AgentTask[GetEmailResult]):
         self._email_update_speech_handle: SpeechHandle | None = None
 
     async def on_enter(self) -> None:
-        self.session.generate_reply(
-            instructions=(
-                "Ask the user to provide an email address. If you already have it, ask for confirmation.\n"
-                "Do not call `decline_email_capture`"
-            )
-        )
+        self.session.generate_reply(instructions="Ask the user to provide an email address.")
 
     @function_tool
     async def update_email_address(self, email: str, ctx: RunContext) -> str:
@@ -104,7 +103,7 @@ class GetEmailTask(AgentTask[GetEmailResult]):
             f"Prompt the user for confirmation, do not call `confirm_email_address` directly"
         )
 
-    @function_tool
+    @function_tool(flags=ToolFlag.IGNORE_ON_ENTER)
     async def confirm_email_address(self, ctx: RunContext) -> None:
         """Validates/confirms the email address provided by the user."""
         await ctx.wait_for_playout()
@@ -120,7 +119,7 @@ class GetEmailTask(AgentTask[GetEmailResult]):
         if not self.done():
             self.complete(GetEmailResult(email_address=self._current_email))
 
-    @function_tool
+    @function_tool(flags=ToolFlag.IGNORE_ON_ENTER)
     async def decline_email_capture(self, reason: str) -> None:
         """Handles the case when the user explicitly declines to provide an email address.
 
