@@ -1,26 +1,14 @@
 from __future__ import annotations
 
 from multiprocessing import current_process
-from types import TracebackType
 
 if current_process().name == "inference_proc":
     import signal
-    import sys
 
     # ignore signals in the inference process (the parent process will handle them)
     signal.signal(signal.SIGINT, signal.SIG_IGN)
     signal.signal(signal.SIGTERM, signal.SIG_IGN)
 
-    def _no_traceback_excepthook(
-        exc_type: type[BaseException], exc_val: BaseException, traceback: TracebackType | None
-    ) -> None:
-        if isinstance(exc_val, KeyboardInterrupt):
-            return
-        sys.__excepthook__(exc_type, exc_val, traceback)
-
-    sys.excepthook = _no_traceback_excepthook
-
-
 import asyncio
 import math
 import socket
@@ -48,18 +36,12 @@ def proc_main(args: ProcStartArgs) -> None:
 
     inf_proc = _InferenceProc(args.runners)
 
-    client = _ProcClient(
-        args.mp_cch,
-        args.log_cch,
-        inf_proc.initialize,
-        inf_proc.entrypoint,
-    )
-
-    client.initialize_logger()
+    client = _ProcClient(args.mp_cch, args.log_cch, inf_proc.initialize, inf_proc.entrypoint)
     try:
         client.initialize()
     except Exception:
         return  # initialization failed, exit (initialize will send an error to the worker)
+
     client.run()
 
 
@@ -113,13 +95,7 @@ class _InferenceProc:
             data = await loop.run_in_executor(
                 self._executor, self._runners[msg.method].run, msg.data
             )
-            await self._client.send(
-                proto.InferenceResponse(
-                    request_id=msg.request_id,
-                    data=data,
-                )
-            )
-
+            await self._client.send(proto.InferenceResponse(request_id=msg.request_id, data=data))
         except Exception as e:
             logger.exception("error running inference")
             await self._client.send(
