@@ -24,6 +24,7 @@ class ProcJobExecutor(SupervisedProc):
         *,
         initialize_process_fnc: Callable[[JobProcess], Any],
         job_entrypoint_fnc: Callable[[JobContext], Awaitable[None]],
+        session_end_fnc: Callable[[JobContext], Awaitable[None]] | None,
         inference_executor: InferenceExecutor | None,
         initialize_timeout: float,
         close_timeout: float,
@@ -54,6 +55,7 @@ class ProcJobExecutor(SupervisedProc):
         self._running_job: RunningJobInfo | None = None
         self._initialize_process_fnc = initialize_process_fnc
         self._job_entrypoint_fnc = job_entrypoint_fnc
+        self._session_end_fnc = session_end_fnc
         self._inference_executor = inference_executor
         self._inference_tasks: list[asyncio.Task[None]] = []
         self._id = shortuuid("PCEXEC_")
@@ -85,6 +87,7 @@ class ProcJobExecutor(SupervisedProc):
         proc_args = ProcStartArgs(
             initialize_process_fnc=self._initialize_process_fnc,
             job_entrypoint_fnc=self._job_entrypoint_fnc,
+            session_end_fnc=self._session_end_fnc,
             log_cch=log_cch,
             mp_cch=cch,
             user_arguments=self._user_args,
@@ -156,5 +159,6 @@ class ProcJobExecutor(SupervisedProc):
 
         if self._running_job:
             extra["job_id"] = self._running_job.job.id
+            extra["room_id"] = self._running_job.job.room.sid
 
         return extra
