@@ -64,6 +64,10 @@ class LogQueueHandler(logging.Handler):
         self._send_thread = threading.Thread(target=self._forward_logs, name="ipc_log_forwarder")
         self._send_thread.start()
 
+    @property
+    def thread(self) -> threading.Thread:
+        return self._send_thread
+
     def _forward_logs(self) -> None:
         while True:
             serialized_record = self._send_q.get()
