@@ -1,5 +1,6 @@
 from . import remote_chat_context, utils
 from .chat_context import (
+    AgentHandoff,
     AudioContent,
     ChatContent,
     ChatContext,
@@ -9,6 +10,7 @@ from .chat_context import (
     FunctionCall,
     FunctionCallOutput,
     ImageContent,
+    MetricsReport,
 )
 from .fallback_adapter import AvailabilityChangedEvent, FallbackAdapter
 from .llm import (
@@ -57,6 +59,8 @@ __all__ = [
     "FunctionCallOutput",
     "AudioContent",
     "ImageContent",
+    "AgentHandoff",
+    "MetricsReport",
     "ChatItem",
     "ChoiceDelta",
     "ChatChunk",
