@@ -19,7 +19,7 @@ from collections.abc import Sequence
 from typing import TYPE_CHECKING, Annotated, Any, Literal, Union, overload
 
 from pydantic import BaseModel, Field, PrivateAttr, TypeAdapter
-from typing_extensions import TypeAlias
+from typing_extensions import TypeAlias, TypedDict
 
 from livekit import rtc
 
@@ -30,7 +30,7 @@ from ..utils.misc import is_given
 from . import _provider_format
 
 if TYPE_CHECKING:
-    from ..llm import FunctionTool, RawFunctionTool
+    from ..llm import LLM, FunctionTool, RawFunctionTool
 
 
 class ImageContent(BaseModel):
@@ -62,7 +62,7 @@ class ImageContent(BaseModel):
     # With an external URL
     chat_image = ImageContent(image="https://example.com/image.jpg")
     ```
-    """  # noqa: E501
+    """
 
     id: str = Field(default_factory=lambda: utils.shortuuid("img_"))
     """
@@ -105,6 +105,49 @@ class AudioContent(BaseModel):
 ChatRole: TypeAlias = Literal["developer", "system", "user", "assistant"]
 
 
+# The metrics are stored in a dict, since some fields may not be relevant
+# in certain context (e.g., text-only mode or when using a speech-to-speech model).
+class MetricsReport(TypedDict, total=False):
+    started_speaking_at: float
+    stopped_speaking_at: float
+
+    transcription_delay: float
+    """Time taken to obtain the transcript after the end of the user's speech
+
+    User `ChatMessage` only
+    """
+
+    end_of_turn_delay: float
+    """Amount of time between the end of speech and the decision to end the user's turn
+
+    User `ChatMessage` only
+    """
+
+    on_user_turn_completed_delay: float
+    """Time taken to invoke the developer's `Agent.on_user_turn_completed` callback.
+
+    User `ChatMessage` only
+    """
+
+    llm_node_ttft: float
+    """Time taken for the `llm_node` to return the first token
+
+    Assistant `ChatMessage` only
+    """
+
+    tts_node_ttfb: float
+    """Time taken for the `tts_node` to return the first chunk of audio (after the first text token has been sent)
+
+    Assistant `ChatMessage` only
+    """
+
+    e2e_latency: float
+    """Time from when the user finished speaking to when the agent began responding
+
+    Assistant `ChatMessage` only
+    """
+
+
 class ChatMessage(BaseModel):
     id: str = Field(default_factory=lambda: utils.shortuuid("item_"))
     type: Literal["message"] = "message"
@@ -112,8 +155,10 @@ class ChatMessage(BaseModel):
     content: list[ChatContent]
     interrupted: bool = False
     transcript_confidence: float | None = None
-    hash: bytes | None = None
+    extra: dict[str, Any] = Field(default_factory=dict)
+    metrics: MetricsReport = Field(default_factory=lambda: MetricsReport())
     created_at: float = Field(default_factory=time.time)
+    hash: bytes | None = Field(default=None, deprecated="hash is deprecated")
 
     @property
     def text_content(self) -> str | None:
@@ -150,19 +195,16 @@ class FunctionCallOutput(BaseModel):
     created_at: float = Field(default_factory=time.time)
 
 
-""""
 class AgentHandoff(BaseModel):
     id: str = Field(default_factory=lambda: utils.shortuuid("item_"))
     type: Literal["agent_handoff"] = Field(default="agent_handoff")
     old_agent_id: str | None
     new_agent_id: str
-    old_agent: Agent | None = Field(exclude=True)
-    new_agent: Agent | None = Field(exclude=True)
     created_at: float = Field(default_factory=time.time)
-"""
+
 
 ChatItem = Annotated[
-    Union[ChatMessage, FunctionCall, FunctionCallOutput], Field(discriminator="type")
+    Union[ChatMessage, FunctionCall, FunctionCallOutput, AgentHandoff], Field(discriminator="type")
 ]
 
 
@@ -190,6 +232,8 @@ class ChatContext:
         id: NotGivenOr[str] = NOT_GIVEN,
         interrupted: NotGivenOr[bool] = NOT_GIVEN,
         created_at: NotGivenOr[float] = NOT_GIVEN,
+        metrics: NotGivenOr[MetricsReport] = NOT_GIVEN,
+        extra: NotGivenOr[dict[str, Any]] = NOT_GIVEN,
     ) -> ChatMessage:
         kwargs: dict[str, Any] = {}
         if is_given(id):
@@ -198,6 +242,10 @@ class ChatContext:
             kwargs["interrupted"] = interrupted
         if is_given(created_at):
             kwargs["created_at"] = created_at
+        if is_given(metrics):
+            kwargs["metrics"] = metrics
+        if is_given(extra):
+            kwargs["extra"] = extra
 
         if isinstance(content, str):
             message = ChatMessage(role=role, content=[content], **kwargs)
@@ -297,7 +345,7 @@ class ChatContext:
         )
 
         new_items = self._items[-max_items:]
-        # chat ctx shouldn't start with function_call or function_call_output
+        # chat_ctx shouldn't start with function_call or function_call_output
         while new_items and new_items[0].type in [
             "function_call",
             "function_call_output",
@@ -453,6 +501,93 @@ class ChatContext:
 
         return 0
 
+    async def summarize(
+        self,
+        llm_v: LLM,
+        *,
+        keep_last_turns: int = 2,
+    ) -> ChatContext:
+        to_summarize: list[ChatMessage] = []
+        for item in self.items:
+            if item.type != "message":
+                continue
+            if item.role not in ("user", "assistant"):
+                continue
+            if item.extra.get("is_summary") is True:  # avoid making summary of summaries
+                continue
+
+            text = (item.text_content or "").strip()
+            if text:
+                to_summarize.append(item)
+        if not to_summarize:
+            return self
+
+        tail_n = max(0, min(len(to_summarize), keep_last_turns * 2))
+        if tail_n == 0:
+            head, tail = to_summarize, []
+        else:
+            head, tail = to_summarize[:-tail_n], to_summarize[-tail_n:]
+
+        if not head:
+            return self
+
+        source_text = "\n".join(f"{m.role}: {(m.text_content or '').strip()}" for m in head).strip()
+        if not source_text:
+            return self
+
+        chat_ctx = ChatContext()
+        chat_ctx.add_message(
+            role="system",
+            content=(
+                "Compress older chat history into a short, faithful summary.\n"
+                "Focus on user goals, constraints, decisions, key facts/preferences/entities, and pending tasks.\n"
+                "Exclude chit-chat and greetings. Be concise."
+            ),
+        )
+        chat_ctx.add_message(
+            role="user",
+            content=f"Conversation to summarize:\n\n{source_text}",
+        )
+
+        chunks: list[str] = []
+        async for chunk in llm_v.chat(chat_ctx=chat_ctx):
+            if chunk.delta and chunk.delta.content:
+                chunks.append(chunk.delta.content)
+
+        summary = "".join(chunks).strip()
+        if not summary:
+            return self
+
+        tail_start_ts = tail[0].created_at if tail else float("inf")
+
+        preserved: list[ChatItem] = []
+        for it in self.items:
+            if (
+                it.type in ("function_call", "function_call_output")
+                and it.created_at < tail_start_ts
+            ):
+                continue
+
+            if it.type == "message" and it.role in ("user", "assistant"):
+                continue
+
+            preserved.append(it)
+
+        self._items = preserved
+
+        created_at_hint = (tail[0].created_at - 1e-6) if tail else (head[-1].created_at + 1e-6)
+        self.add_message(
+            role="assistant",
+            content=f"[history summary]\n{summary}",
+            created_at=created_at_hint,
+            extra={"is_summary": True},
+        )
+
+        for msg in tail:
+            self._items.append(msg)
+
+        return self
+
     @classmethod
     def from_dict(cls, data: dict[str, Any]) -> ChatContext:
         item_adapter = TypeAdapter(list[ChatItem])
