@@ -7,7 +7,7 @@ from abc import ABC, abstractmethod
 from collections.abc import AsyncIterable, AsyncIterator
 from datetime import datetime, timezone
 from types import TracebackType
-from typing import Any, Generic, Literal, TypeVar, Union
+from typing import Any, ClassVar, Generic, Literal, TypeVar, Union
 
 from opentelemetry import trace
 from opentelemetry.util.types import AttributeValue
@@ -21,7 +21,6 @@ from .._exceptions import APIConnectionError, APIError
 from ..log import logger
 from ..metrics import LLMMetrics
 from ..telemetry import trace_types, tracer, utils as telemetry_utils
-from ..telemetry.traces import _chat_ctx_to_otel_events
 from ..types import (
     DEFAULT_API_CONNECT_OPTIONS,
     NOT_GIVEN,
@@ -147,6 +146,8 @@ class LLM(
 
 
 class LLMStream(ABC):
+    _llm_request_span_name: ClassVar[str] = "llm_request"
+
     def __init__(
         self,
         llm: LLM,
@@ -167,7 +168,11 @@ class LLMStream(ABC):
             self._metrics_monitor_task(monitor_aiter), name="LLM._metrics_task"
         )
 
-        self._task = asyncio.create_task(self._main_task())
+        async def _traceable_main_task() -> None:
+            with tracer.start_as_current_span(self._llm_request_span_name, end_on_exit=False):
+                await self._main_task()
+
+        self._task = asyncio.create_task(_traceable_main_task(), name="LLM._main_task")
         self._task.add_done_callback(lambda _: self._event_ch.close())
 
         self._llm_request_span: trace.Span | None = None
@@ -175,12 +180,9 @@ class LLMStream(ABC):
     @abstractmethod
     async def _run(self) -> None: ...
 
-    @tracer.start_as_current_span("llm_request", end_on_exit=False)
     async def _main_task(self) -> None:
         self._llm_request_span = trace.get_current_span()
         self._llm_request_span.set_attribute(trace_types.ATTR_GEN_AI_REQUEST_MODEL, self._llm.model)
-        for name, attributes in _chat_ctx_to_otel_events(self._chat_ctx):
-            self._llm_request_span.add_event(name, attributes)
 
         for i in range(self._conn_options.max_retry + 1):
             try:
