@@ -12,6 +12,7 @@ from typing import (
     Any,
     Callable,
     Union,
+    cast,
     get_args,
     get_origin,
     get_type_hints,
@@ -134,7 +135,12 @@ class SerializedImage:
     external_url: str | None = None
 
 
-def serialize_image(image: ImageContent) -> SerializedImage:
+def serialize_image(image: ImageContent, *, use_cache: bool = True) -> SerializedImage:
+    cache_key = "serialized_image"  # TODO(long): use hash of encoding options if available
+    if use_cache and cache_key in image._cache:
+        return cast(SerializedImage, image._cache[cache_key])
+
+    serialized_image: SerializedImage
     if isinstance(image.image, str):
         if image.image.startswith("data:"):
             header, b64_data = image.image.split(",", 1)
@@ -154,13 +160,13 @@ def serialize_image(image: ImageContent) -> SerializedImage:
                     f"Unsupported mime_type {mime_type}. Must be jpeg, png, webp, or gif"
                 )
 
-            return SerializedImage(
+            serialized_image = SerializedImage(
                 data_bytes=encoded_data,
                 mime_type=mime_type,
                 inference_detail=image.inference_detail,
             )
         else:
-            return SerializedImage(
+            serialized_image = SerializedImage(
                 mime_type=image.mime_type,
                 inference_detail=image.inference_detail,
                 external_url=image.image,
@@ -176,12 +182,17 @@ def serialize_image(image: ImageContent) -> SerializedImage:
             )
         encoded_data = images.encode(image.image, opts)
 
-        return SerializedImage(
+        serialized_image = SerializedImage(
             data_bytes=encoded_data,
             mime_type="image/jpeg",
             inference_detail=image.inference_detail,
         )
-    raise ValueError("Unsupported image type")
+    else:
+        raise ValueError("Unsupported image type")
+
+    if use_cache:
+        image._cache[cache_key] = serialized_image
+    return serialized_image
 
 
 def build_legacy_openai_schema(
@@ -407,7 +418,7 @@ def _is_optional_type(hint: Any) -> bool:
 
 def _shallow_model_dump(model: BaseModel, *, by_alias: bool = False) -> dict[str, Any]:
     result = {}
-    for name, field in model.model_fields.items():
+    for name, field in model.__class__.model_fields.items():
         key = field.alias if by_alias and field.alias else name
         result[key] = getattr(model, name)
     return result
