@@ -37,7 +37,22 @@ def log_metrics(metrics: AgentMetrics, *, logger: logging.Logger | None = None)
                 "ttft": round(metrics.ttft, 2),
                 "input_tokens": metrics.input_tokens,
                 "cached_input_tokens": metrics.input_token_details.cached_tokens,
+                "input_text_tokens": metrics.input_token_details.text_tokens,
+                "input_cached_text_tokens": metrics.input_token_details.cached_tokens_details.text_tokens
+                if metrics.input_token_details.cached_tokens_details
+                else 0,
+                "input_image_tokens": metrics.input_token_details.image_tokens,
+                "input_cached_image_tokens": metrics.input_token_details.cached_tokens_details.image_tokens
+                if metrics.input_token_details.cached_tokens_details
+                else 0,
+                "input_audio_tokens": metrics.input_token_details.audio_tokens,
+                "input_cached_audio_tokens": metrics.input_token_details.cached_tokens_details.audio_tokens
+                if metrics.input_token_details.cached_tokens_details
+                else 0,
                 "output_tokens": metrics.output_tokens,
+                "output_text_tokens": metrics.output_token_details.text_tokens,
+                "output_audio_tokens": metrics.output_token_details.audio_tokens,
+                "output_image_tokens": metrics.output_token_details.image_tokens,
                 "total_tokens": metrics.total_tokens,
                 "tokens_per_second": round(metrics.tokens_per_second, 2),
             },
