@@ -1,4 +1,5 @@
 from .fallback_adapter import AvailabilityChangedEvent, FallbackAdapter
+from .multi_speaker_adapter import MultiSpeakerAdapter
 from .stream_adapter import StreamAdapter, StreamAdapterWrapper
 from .stt import (
     STT,
@@ -26,6 +27,7 @@ __all__ = [
     "FallbackAdapter",
     "AvailabilityChangedEvent",
     "STTError",
+    "MultiSpeakerAdapter",
 ]
 
 # Cleanup docs of unexported modules
