@@ -34,6 +34,7 @@ class DataStreamAudioOutput(AudioOutput):
         destination_identity: str,
         sample_rate: int | None = None,
         wait_remote_track: rtc.TrackKind.ValueType | None = None,
+        clear_buffer_timeout: float | None = 2.0,
     ):
         super().__init__(
             label="DataStreamIO",
@@ -52,6 +53,11 @@ class DataStreamAudioOutput(AudioOutput):
         self._lock = asyncio.Lock()
         self._start_atask: asyncio.Task | None = None
 
+        # a playback finished event is expected after the clear buffer rpc is performed
+        # if not received after the timeout, we still mark the playout is done to avoid deadlock
+        self._clear_buffer_timeout = clear_buffer_timeout
+        self._clear_buffer_timeout_handler: asyncio.TimerHandle | None = None
+
         def _on_room_connected(fut: asyncio.Future[None]) -> None:
             if not self._start_atask and not fut.cancelled() and not fut.exception():
                 # register the rpc method right after the room is connected
@@ -148,13 +154,7 @@ class DataStreamAudioOutput(AudioOutput):
         if not self._started:
             return
 
-        task = asyncio.create_task(
-            self._room.local_participant.perform_rpc(
-                destination_identity=self._destination_identity,
-                method=RPC_CLEAR_BUFFER,
-                payload="",
-            )
-        )
+        task = asyncio.create_task(self._clear_buffer_task(self._pushed_duration))
         self._tasks.add(task)
         task.add_done_callback(self._tasks.discard)
 
@@ -168,6 +168,33 @@ class DataStreamAudioOutput(AudioOutput):
             "disable `AgentSession.resume_false_interruption` if you are using an avatar plugin."
         )
 
+    async def _clear_buffer_task(self, pushed_duration: float) -> None:
+        timeout = self._clear_buffer_timeout
+        try:
+            await self._room.local_participant.perform_rpc(
+                destination_identity=self._destination_identity,
+                method=RPC_CLEAR_BUFFER,
+                payload="",
+            )
+        except Exception as e:
+            logger.error("failed to perform clear buffer rpc", exc_info=e)
+            timeout = 0  # mark playout done immediately if clear buffer rpc fails
+
+        def _on_timeout() -> None:
+            logger.warning(
+                "didn't receive playback finished event after clear buffer, marking playout as done arbitrarily"
+            )
+            self.on_playback_finished(playback_position=pushed_duration, interrupted=True)
+            self._reset_playback_count()
+
+        if self._clear_buffer_timeout_handler:
+            self._clear_buffer_timeout_handler.cancel()
+
+        if timeout is not None:
+            self._clear_buffer_timeout_handler = asyncio.get_event_loop().call_later(
+                timeout, _on_timeout
+            )
+
     def _handle_playback_finished(self, data: rtc.RpcInvocationData) -> str:
         if data.caller_identity != self._destination_identity:
             logger.warning(
@@ -184,6 +211,10 @@ class DataStreamAudioOutput(AudioOutput):
             extra={"caller_identity": data.caller_identity},
         )
 
+        if self._clear_buffer_timeout_handler:
+            self._clear_buffer_timeout_handler.cancel()
+            self._clear_buffer_timeout_handler = None
+
         event = PlaybackFinishedEvent(**json.loads(data.payload))
         self.on_playback_finished(
             playback_position=event.playback_position,
