@@ -10,7 +10,7 @@ from livekit import rtc
 
 from .. import llm, stt
 from ..log import logger
-from ..types import NOT_GIVEN, NotGivenOr
+from ..types import NOT_GIVEN, FlushSentinel, NotGivenOr
 from .agent import ModelSettings
 
 # TODO(theomonnom): can those types be simplified?
@@ -24,8 +24,14 @@ STTNode = Callable[
 LLMNode = Callable[
     [llm.ChatContext, list[Union[llm.FunctionTool, llm.RawFunctionTool]], ModelSettings],
     Union[
-        Optional[Union[AsyncIterable[Union[llm.ChatChunk, str]], str, llm.ChatChunk]],
-        Awaitable[Optional[Union[AsyncIterable[Union[llm.ChatChunk, str]], str, llm.ChatChunk]]],
+        Optional[
+            Union[AsyncIterable[Union[llm.ChatChunk, str, FlushSentinel]], str, llm.ChatChunk]
+        ],
+        Awaitable[
+            Optional[
+                Union[AsyncIterable[Union[llm.ChatChunk, str, FlushSentinel]], str, llm.ChatChunk]
+            ]
+        ],
     ],
 ]
 TTSNode = Callable[
@@ -77,11 +83,11 @@ class AudioInput:
 
     def on_attached(self) -> None:
         if self.source:
-            self.on_attached()
+            self.source.on_attached()
 
     def on_detached(self) -> None:
         if self.source:
-            self.on_detached()
+            self.source.on_detached()
 
     def __repr__(self) -> str:
         return f"{self.__class__.__name__}(label={self.label!r}, source={self.source!r})"
@@ -111,11 +117,11 @@ class VideoInput:
 
     def on_attached(self) -> None:
         if self.source:
-            self.on_attached()
+            self.source.on_attached()
 
     def on_detached(self) -> None:
         if self.source:
-            self.on_detached()
+            self.source.on_detached()
 
     def __repr__(self) -> str:
         return f"{self.__class__.__name__}(label={self.label!r}, source={self.source!r})"
@@ -227,6 +233,10 @@ class AudioOutput(ABC, rtc.EventEmitter[Literal["playback_finished"]]):
 
         return self.__last_playback_ev
 
+    def _reset_playback_count(self) -> None:
+        self.__playback_segments_count = 0
+        self.__playback_finished_count = 0
+
     @property
     def sample_rate(self) -> int | None:
         """The sample rate required by the audio sink, if None, any sample rate is accepted"""
