@@ -4,18 +4,22 @@ import asyncio
 import contextlib
 import queue
 import threading
+import time
 from collections.abc import AsyncIterator
-from typing import Any, Callable
+from pathlib import Path
+from typing import TYPE_CHECKING, Any, Callable
 
 import av
 import numpy as np
 
 from livekit import rtc
-from livekit.agents.voice.agent_session import AgentSession
 
 from ...log import logger
 from .. import io
 
+if TYPE_CHECKING:
+    from ..agent_session import AgentSession
+
 # the recorder currently assume the input is a continous uninterrupted audio stream
 
 
@@ -41,8 +45,9 @@ class RecorderIO:
         self._loop = loop or asyncio.get_event_loop()
         self._lock = asyncio.Lock()
         self._close_fut: asyncio.Future[None] = self._loop.create_future()
+        self._output_path: Path | None = None
 
-    async def start(self, *, output_path: str) -> None:
+    async def start(self, *, output_path: str | Path) -> None:
         async with self._lock:
             if self._started:
                 return
@@ -53,12 +58,14 @@ class RecorderIO:
                     "must be called before starting the recorder."
                 )
 
-            self._output_path = output_path
+            self._output_path = Path(output_path)
             self._started = True
             self._close_fut = self._loop.create_future()
             self._forward_atask = asyncio.create_task(self._forward_task())
 
-            thread = threading.Thread(target=self._encode_thread, daemon=True)
+            thread = threading.Thread(
+                target=self._encode_thread, daemon=True, name="recorder_io_encode_thread"
+            )
             thread.start()
 
     async def aclose(self) -> None:
@@ -85,6 +92,23 @@ class RecorderIO:
     def recording(self) -> bool:
         return self._started
 
+    @property
+    def output_path(self) -> Path | None:
+        return self._output_path
+
+    @property
+    def recording_started_at(self) -> float | None:
+        in_t = self._in_record.started_wall_time if self._in_record else None
+        out_t = self._out_record.started_wall_time if self._out_record else None
+
+        if in_t is None:
+            return out_t
+
+        if out_t is None:
+            return in_t
+
+        return min(in_t, out_t)
+
     def _write_cb(self, buf: list[rtc.AudioFrame]) -> None:
         assert self._in_record is not None
 
@@ -111,6 +135,9 @@ class RecorderIO:
         GROW_FACTOR = 1.5
         INV_INT16 = 1.0 / 32768.0
 
+        assert self._output_path is not None
+        self._output_path.parent.mkdir(parents=True, exist_ok=True)
+
         container = av.open(self._output_path, mode="w", format="ogg")
         stream: av.AudioStream = container.add_stream(
             "opus", rate=self._sample_rate, layout="stereo"
@@ -206,8 +233,10 @@ class RecorderIO:
                         len_right = len_left
 
                 max_len = max(len_left, len_right)
-                stereo_slice = stereo_buf[:, :max_len]
+                if max_len <= 0:
+                    continue
 
+                stereo_slice = stereo_buf[:, :max_len]
                 av_frame = av.AudioFrame.from_ndarray(stereo_slice, format="fltp", layout="stereo")
                 av_frame.sample_rate = self._sample_rate
 
@@ -227,6 +256,11 @@ class RecorderAudioInput(io.AudioInput):
         self.__audio_input = source
         self.__recording_io = recording_io
         self.__acc_frames: list[rtc.AudioFrame] = []
+        self.__started_time: None | float = None
+
+    @property
+    def started_wall_time(self) -> float | None:
+        return self.__started_time
 
     def take_buf(self) -> list[rtc.AudioFrame]:
         frames = self.__acc_frames
@@ -240,14 +274,13 @@ class RecorderAudioInput(io.AudioInput):
         frame = await self.__audio_input.__anext__()
 
         if self.__recording_io.recording:
+            if self.__started_time is None:
+                self.__started_time = time.time()
+
             self.__acc_frames.append(frame)
 
         return frame
 
-    def on_attached(self) -> None: ...
-
-    def on_detached(self) -> None: ...
-
 
 class RecorderAudioOutput(io.AudioOutput):
     def __init__(
@@ -266,6 +299,15 @@ class RecorderAudioOutput(io.AudioOutput):
         self.__recording_io = recording_io
         self.__write = write_fnc
         self.__acc_frames: list[rtc.AudioFrame] = []
+        self.__started_time: None | float = None
+
+    @property
+    def started_wall_time(self) -> float | None:
+        return self.__started_time
+
+    @property
+    def recorder_io(self) -> RecorderIO:
+        return self.__recording_io
 
     @property
     def has_pending_data(self) -> bool:
@@ -292,9 +334,9 @@ class RecorderAudioOutput(io.AudioOutput):
         for frame in self.__acc_frames:
             if frame.duration + acc_dur > playback_position:
                 duration_needed = playback_position - acc_dur
-                samples_needed = int(duration_needed * frame.sample_rate) * frame.num_channels
+                samples_needed = int(duration_needed * frame.sample_rate)  #  * frame.num_channels
                 truncated_frame = rtc.AudioFrame(
-                    data=frame.data[:samples_needed],
+                    data=frame.data[: samples_needed * frame.num_channels],
                     num_channels=frame.num_channels,
                     samples_per_channel=samples_needed,
                     sample_rate=frame.sample_rate,
@@ -314,6 +356,9 @@ class RecorderAudioOutput(io.AudioOutput):
         await super().capture_frame(frame)
 
         if self.__recording_io.recording:
+            if self.__started_time is None:
+                self.__started_time = time.time()
+
             self.__acc_frames.append(frame)
 
         if self.next_in_chain:
